/* (C) 2022 YiRing, Inc. */
package com.yiring.app.domain.location;

import com.yiring.common.annotation.FieldMapping;
import java.io.Serial;
import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.HashSet;
import java.util.Set;
import javax.persistence.*;
import lombok.*;
import lombok.experimental.FieldDefaults;
import lombok.experimental.FieldNameConstants;
import org.hibernate.annotations.Comment;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Type;
import org.hibernate.snowflake.SnowflakeId;
import org.locationtech.jts.geom.Geometry;

/**
 * 围栏
 * 引用: 定位平台接口规范V3.0.1 #7.1
 *
 * @author Jim
 * @version 0.1
 * 2022/4/7 16:26
 */

@Getter
@Setter
@ToString
@Builder
@NoArgsConstructor
@AllArgsConstructor
@FieldNameConstants
@FieldDefaults(level = AccessLevel.PRIVATE)
@Entity
@Table(name = "BS_LOCATION_FENCE", indexes = { @Index(columnList = "linkId"), @Index(columnList = "mode") })
@Comment("围栏")
public class LocationFence implements Serializable {

    @Serial
    private static final long serialVersionUID = 4155868702188991300L;

    @Comment("主键")
    @Id
    @GeneratedValue(generator = SnowflakeId.GENERATOR)
    @GenericGenerator(name = SnowflakeId.GENERATOR, strategy = SnowflakeId.Strategy.LONG)
    Long id;

    /**
     * 数据来源于【真源人员定位系统 - 电子围栏】
     * 作用: 用于双向联动进行数据同步
     */
    @FieldMapping("id")
    @Comment("外链主键")
    Long linkId;

    @Comment("地图 ID")
    Long areaId;

    @FieldMapping("entityTypes")
    @Comment("围栏模型")
    @Enumerated(EnumType.STRING)
    Mode mode;

    @Comment("坐标信息（x,y,r）")
    @Column(columnDefinition = "JSON")
    String shape;

    @Comment("形状信息（circle: 圆形, polygon: 多边形）")
    String shapeType;

    @Comment("空间信息")
    @Type(type = "jts_geometry")
    @Column(columnDefinition = "geometry")
    Geometry geometry;

    @Comment("半径，米/单位（圆形围栏）")
    Double radius;

    @Comment("超时时间（秒）")
    Integer overtime;

    @Comment("消抖时间（秒）")
    Integer threshold;

    @Comment("信标集合")
    @Builder.Default
    @ManyToMany
    @ToString.Exclude
    Set<LocationBeacon> beacons = new HashSet<>(0);

    @Comment("创建时间")
    LocalDateTime createTime;

    @SuppressWarnings({ "unused" })
    public enum Mode {
        NORMAL("标准"),
        DANGER("危险区域");

        @Getter
        final String text;

        Mode(String text) {
            this.text = text;
        }
    }
}
