/* (C) 2022 YiRing, Inc. */
package com.yiring.app.domain.location;

import com.alibaba.fastjson.JSON;
import com.vladmihalcea.hibernate.type.json.JsonType;
import com.yiring.auth.domain.user.User;
import java.io.Serial;
import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.HashSet;
import java.util.Objects;
import java.util.Set;
import javax.persistence.*;
import lombok.*;
import lombok.experimental.FieldDefaults;
import lombok.experimental.FieldNameConstants;
import org.hibernate.Hibernate;
import org.hibernate.annotations.Comment;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Type;
import org.hibernate.annotations.TypeDef;
import org.hibernate.snowflake.SnowflakeId;
import org.locationtech.jts.geom.Point;

/**
 * 定位数据
 * 引用: 定位平台接口规范V3.0.1 #6
 *
 * @author Jim
 * @version 0.1
 * 2022/4/7 10:21
 */

@Getter
@Setter
@ToString
@Builder
@NoArgsConstructor
@AllArgsConstructor
@FieldNameConstants
@FieldDefaults(level = AccessLevel.PRIVATE)
@Entity
@TypeDef(name = "json", typeClass = JsonType.class)
@Table(name = "BS_LOCATION_LOG", indexes = { @Index(columnList = "time"), @Index(columnList = "silent") })
@Comment("定位数据")
public class LocationLog implements Serializable {

    @Serial
    private static final long serialVersionUID = 3467455881020691989L;

    @Comment("主键")
    @Id
    @GeneratedValue(generator = SnowflakeId.GENERATOR)
    @GenericGenerator(name = SnowflakeId.GENERATOR, strategy = SnowflakeId.Strategy.LONG)
    Long id;

    /**
     * 来源于定位数据产生时刻标签所属的人员
     */
    @Comment("用户")
    @ManyToOne
    @JoinColumn(name = "user_id")
    User user;

    /**
     * 来源于定位标签定位时刻的人员状态
     */
    @Comment("用户状态")
    String status;

    @Comment("标签")
    @ManyToOne
    @JoinColumn(name = "tag_id")
    LocationTag tag;

    @Comment("时间")
    @Column(nullable = false)
    LocalDateTime time;

    @Comment("经度")
    BigDecimal lon;

    @Comment("纬度")
    BigDecimal lat;

    @Comment("海拔高度（m）")
    BigDecimal altitude;

    @Comment("坐标点信息")
    @Type(type = "jts_geometry")
    @Column(columnDefinition = "point")
    Point point;

    @Comment("信标集合")
    @Builder.Default
    @ManyToMany
    @ToString.Exclude
    Set<LocationBeacon> beacons = new HashSet<>(0);

    @Comment("围栏集合")
    @Builder.Default
    @ManyToMany
    @ToString.Exclude
    Set<LocationFence> fences = new HashSet<>(0);

    @Comment("静止/运动")
    Boolean silent;

    @Comment("电量")
    Integer volt;

    @Comment("电量单位")
    String voltUnit;

    @Comment("原始数据")
    @org.hibernate.annotations.Type(type = "json")
    @Column(columnDefinition = "json")
    JSON raw;

    @Comment("创建时间")
    LocalDateTime createTime;

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || Hibernate.getClass(this) != Hibernate.getClass(o)) return false;
        LocationLog locationLog = (LocationLog) o;
        return id != null && Objects.equals(id, locationLog.id);
    }

    @Override
    public int hashCode() {
        return getClass().hashCode();
    }
}
