/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.visitor.impl;

import cn.hutool.core.util.StrUtil;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.yiring.app.domain.visitor.Visitor;
import com.yiring.app.domain.visitor.VisitorRepository;
import com.yiring.app.excel.visitor.VisitorExportExcel;
import com.yiring.app.param.visitor.VisitorInfoParam;
import com.yiring.app.param.visitor.VisitorListParam;
import com.yiring.app.param.visitor.VisitorParam;
import com.yiring.app.param.visitor.VisitorQueryParam;
import com.yiring.app.service.visitor.VisitorService;
import com.yiring.app.vo.visitor.VisitorVo;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.vo.PageVo;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.*;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author tzl
 * 2022/4/19 14:12
 */
@Slf4j
@Transactional(rollbackFor = RuntimeException.class)
@Service
public class VisitorServiceImpl implements VisitorService {

    @Resource
    VisitorRepository visitorRepository;

    @Override
    public Result<String> saveVisitor(VisitorParam visitorParam) {
        if (hasVisitorMobile(visitorParam.getVisitorMobile())) {
            return Result.no(Status.BAD_REQUEST, "您输入的手机号码已经预约");
        }
        if (hasCarNum(visitorParam.getVisitorCarNum())) {
            return Result.no(Status.BAD_REQUEST, "您输入的身份证号码已经预约");
        }
        //设置创建时间和默认状态
        Visitor visitor = Visitor.builder().createTime(LocalDateTime.now()).build();
        BeanUtils.copyProperties(visitorParam, visitor);
        visitorRepository.saveAndFlush(visitor);
        return Result.ok();
    }

    @Override
    public Result<String> saveVisitor(VisitorListParam visitorListParam) {
        List<VisitorInfoParam> visitorInfos = visitorListParam.getVisitorInfos();
        List<String> mobileList = new ArrayList<>();
        List<String> carList = new ArrayList<>();

        visitorInfos.forEach(visitorInfoParam -> {
            if (hasVisitorMobile(visitorInfoParam.getVisitorMobile())) {
                mobileList.add(visitorInfoParam.getVisitorMobile());
            }
            if (hasCarNum(visitorInfoParam.getVisitorCarNum())) {
                carList.add(visitorInfoParam.getVisitorCarNum());
            }
        });
        if (mobileList.size() > 0) {
            return Result.no(Status.BAD_REQUEST, mobileList + "手机号已经预约,请勿重复预约");
        }
        if (carList.size() > 0) {
            return Result.no(Status.BAD_REQUEST, carList + "身份证已经预约,请勿重复预约");
        }
        visitorInfos.forEach(visitorInfoParam -> {
            Visitor visitor = new Visitor();
            BeanUtils.copyProperties(visitorInfoParam, visitor);
            visitor.setOrgId(visitorListParam.getOrgId());
            visitor.setNumOfVisitor(visitorListParam.getNumOfVisitor());
            visitor.setReason(visitorListParam.getReason());
            visitor.setIntervieweeId(visitorListParam.getIntervieweeId());
            visitorRepository.saveAndFlush(visitor);
        });
        return Result.ok();
    }

    @Override
    public Result<String> deleteVisitor(IdParam idParam) {
        Optional<Visitor> optional = visitorRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        visitorRepository.delete(optional.get());
        return Result.ok();
    }

    @Override
    public Result<String> updateVisitor(VisitorParam visitorParam, IdParam idParam) {
        Optional<Visitor> optional = visitorRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Visitor visitor = optional.get();
        if (!visitor.getVisitorCarNum().equals(visitorParam.getVisitorCarNum())) {
            //当修改身份证时
            if (hasCarNum(visitorParam.getVisitorCarNum())) {
                return Result.no(Status.BAD_REQUEST, "您输入的身份证号码已经预约");
            }
        }
        if (!visitor.getVisitorMobile().equals(visitorParam.getVisitorMobile())) {
            //当修改手机号码时
            if (hasVisitorMobile(visitorParam.getVisitorMobile())) {
                return Result.no(Status.BAD_REQUEST, "您输入的手机号码已经预约");
            }
        }
        BeanUtils.copyProperties(visitorParam, visitor);
        visitorRepository.saveAndFlush(visitor);
        return Result.ok();
    }

    @Override
    public Result<VisitorVo> selectVisitor(IdParam idParam) {
        Optional<Visitor> optional = visitorRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }

        Visitor visitor = optional.get();
        VisitorVo vo = new VisitorVo();
        BeanUtils.copyProperties(visitor, vo);
        return Result.ok(vo);
    }

    @Override
    public Result<PageVo<VisitorVo>> pageVisitor(VisitorQueryParam visitorQueryParam) {
        PageVo<VisitorVo> resultVo;
        //排序
        Sort sort = Sort.by(Sort.Order.desc(Visitor.Fields.createTime));
        if (Objects.nonNull(visitorQueryParam.getPageNo()) && Objects.nonNull(visitorQueryParam.getPageSize())) {
            //分页
            Pageable pageable = PageRequest.of(visitorQueryParam.getPageNo() - 1, visitorQueryParam.getPageSize());
            Page<Visitor> all = visitorRepository.findAll(condition(visitorQueryParam), pageable);
            List<VisitorVo> data = all
                .get()
                .map(visitor -> {
                    VisitorVo vo = new VisitorVo();
                    BeanUtils.copyProperties(visitor, vo);
                    return vo;
                })
                .collect(Collectors.toList());
            resultVo = PageVo.build(data, all.getTotalElements());
        } else {
            List<Visitor> all = visitorRepository.findAll(condition(visitorQueryParam), sort);
            List<VisitorVo> data = all
                .stream()
                .map(visitor -> {
                    VisitorVo vo = new VisitorVo();
                    BeanUtils.copyProperties(visitor, vo);
                    return vo;
                })
                .collect(Collectors.toList());
            resultVo = PageVo.build(data, data.size());
        }
        return Result.ok(resultVo);
    }

    @Override
    public void exportVisitor(VisitorQueryParam visitorQueryParam, HttpServletResponse response) {
        List<Visitor> visitors = visitorRepository.findAll(condition(visitorQueryParam));
        List<VisitorExportExcel> visitorExportExcels = visitors
            .stream()
            .map(visitor -> {
                VisitorExportExcel visitorExportExcel = new VisitorExportExcel();
                BeanUtils.copyProperties(visitor, visitorExportExcel);
                return visitorExportExcel;
            })
            .toList();

        try (
            DefaultStreamExcelBuilder<VisitorExportExcel> defaultStreamExcelBuilder = DefaultStreamExcelBuilder
                .of(VisitorExportExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .widths(8)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            defaultStreamExcelBuilder.append(visitorExportExcels);

            String fileName = URLEncoder.encode("访客信息.xlsx", StandardCharsets.UTF_8);
            response.setContentType("application/octet-stream");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            OutputStream out = response.getOutputStream();

            Workbook workbook = defaultStreamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("导出访客信息失败: " + e.getMessage());
        }
    }

    @Override
    public Result<String> cardSend(String labelCard, IdParam idParam) {
        Optional<Visitor> optional = visitorRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Visitor visitor = optional.get();
        //状态变更为使用中
        visitor.setLabelCardStatus("2");
        visitor.setCardSendTime(LocalDateTime.now());
        visitor.setLabelCard(labelCard);
        visitorRepository.saveAndFlush(visitor);
        return Result.ok();
    }

    @Override
    public Result<String> cardRec(IdParam idParam) {
        Optional<Visitor> optional = visitorRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Visitor visitor = optional.get();
        visitor.setLabelCardStatus("3");
        visitor.setCardRecTime(LocalDateTime.now());
        visitorRepository.saveAndFlush(visitor);
        return Result.ok();
    }

    public boolean hasCarNum(String carNum) {
        Visitor visitor = Visitor.builder().visitorCarNum(carNum).labelCardStatus("1").build();
        long count = visitorRepository.count(Example.of(visitor));
        Visitor visitor1 = Visitor.builder().visitorCarNum(carNum).labelCardStatus("2").build();
        long count1 = visitorRepository.count(Example.of(visitor1));
        return count + count1 > 0;
    }

    public boolean hasVisitorMobile(String mobile) {
        Visitor visitor = Visitor.builder().visitorMobile(mobile).labelCardStatus("1").build();
        long count = visitorRepository.count(Example.of(visitor));
        Visitor visitor1 = Visitor.builder().visitorMobile(mobile).labelCardStatus("1").build();
        long count1 = visitorRepository.count(Example.of(visitor1));
        return count + count1 > 0;
    }

    public Specification<Visitor> condition(VisitorQueryParam visitorQueryParam) {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            //查询条件
            if (StrUtil.isNotBlank(visitorQueryParam.getVisitorName())) {
                //访客姓名
                list.add(
                    criteriaBuilder.like(
                        root.get(Visitor.Fields.visitorName),
                        "%" + visitorQueryParam.getVisitorName() + "%"
                    )
                );
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getVisitorMobile())) {
                //访客电话
                list.add(
                    criteriaBuilder.like(
                        root.get(Visitor.Fields.visitorMobile),
                        "%" + visitorQueryParam.getVisitorMobile() + "%"
                    )
                );
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getVisitorCarNum())) {
                //访客身份证
                list.add(
                    criteriaBuilder.like(
                        root.get(Visitor.Fields.visitorCarNum),
                        "%" + visitorQueryParam.getVisitorCarNum() + "%"
                    )
                );
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getTaskType())) {
                //是否特殊作业
                list.add(
                    criteriaBuilder.like(root.get(Visitor.Fields.taskType), "%" + visitorQueryParam.getTaskType() + "%")
                );
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getOrgId())) {
                //所属单位查询
                list.add(
                    criteriaBuilder.like(root.get(Visitor.Fields.orgId), "%" + visitorQueryParam.getOrgId() + "%")
                );
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getVisitorSource())) {
                //访客来源
                list.add(
                    criteriaBuilder.like(
                        root.get(Visitor.Fields.visitorSource),
                        "%" + visitorQueryParam.getVisitorSource() + "%"
                    )
                );
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getIntervieweeId())) {
                //被访人
                list.add(
                    criteriaBuilder.equal(root.get(Visitor.Fields.intervieweeId), visitorQueryParam.getIntervieweeId())
                );
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getLabelCard())) {
                //标签卡号查询
                list.add(
                    criteriaBuilder.like(
                        root.get(Visitor.Fields.labelCard),
                        "%" + visitorQueryParam.getLabelCard() + "%"
                    )
                );
            }

            if (StrUtil.isNotBlank(visitorQueryParam.getLabelCardStatus())) {
                //标签卡状态
                list.add(
                    criteriaBuilder.like(
                        root.get(Visitor.Fields.labelCardStatus),
                        "%" + visitorQueryParam.getLabelCardStatus() + "%"
                    )
                );
            }

            if (
                Objects.nonNull(visitorQueryParam.getAppOfVisitTimeStart()) &&
                Objects.nonNull(visitorQueryParam.getAppOfVisitTimeEnd())
            ) {
                //预约到访时间区间段查询
                list.add(
                    criteriaBuilder.between(
                        root.get(Visitor.Fields.appOfVisitTime),
                        visitorQueryParam.getAppOfVisitTimeStart(),
                        visitorQueryParam.getAppOfVisitTimeEnd()
                    )
                );
            }

            if (
                Objects.nonNull(visitorQueryParam.getCardSendTimeStart()) &&
                Objects.nonNull(visitorQueryParam.getCardSendTimeEnd())
            ) {
                //实际到访时间区间段查询
                list.add(
                    criteriaBuilder.between(
                        root.get(Visitor.Fields.cardSendTime),
                        visitorQueryParam.getCardSendTimeStart(),
                        visitorQueryParam.getCardSendTimeEnd()
                    )
                );
            }

            if (
                Objects.nonNull(visitorQueryParam.getCardRecTimeStart()) &&
                Objects.nonNull(visitorQueryParam.getCardRecTimeEnd())
            ) {
                //收卡时间区间段查询
                list.add(
                    criteriaBuilder.between(
                        root.get(Visitor.Fields.cardRecTime),
                        visitorQueryParam.getCardRecTimeStart(),
                        visitorQueryParam.getCardRecTimeEnd()
                    )
                );
            }

            Predicate[] predicates = new Predicate[list.size()];
            query.where(list.toArray(predicates));
            return criteriaBuilder.and(list.toArray(predicates));
        };
    }
}
